@@ -47,7 +47,7 @@
 #include "sudo_auth.h"
 
 #ifndef lint
-__unused static const char rcsid[] = "$Sudo: aix_auth.c,v 1.18.2.7 2009/01/28 16:15:18 millert Exp $";
+__unused static const char rcsid[] = "$Sudo: aix_auth.c,v 1.26 2009/03/01 00:58:41 millert Exp $";
 #endif /* lint */
 
 /*
@@ -81,10 +81,8 @@ aixauth_cleanup(pw, auth)
     struct passwd *pw;
     sudo_auth *auth;
 {
-#ifdef HAVE_UNSETENV
     /* Unset AUTHSTATE as it may not be correct for the runas user. */
     unsetenv("AUTHSTATE");
-#endif
 
     return(AUTH_SUCCESS);
 }
