@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2005 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 1999-2005, 2007-2008 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -13,7 +13,7 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Sudo: sudo_auth.h,v 1.20.2.3 2008/11/06 00:14:58 millert Exp $
+ * $Sudo: sudo_auth.h,v 1.28 2008/12/02 17:30:39 millert Exp $
  */
 
 #ifndef SUDO_AUTH_H
@@ -22,13 +22,14 @@
 /* Auth function return values.  */
 #define AUTH_SUCCESS	0
 #define AUTH_FAILURE	1
-#define AUTH_FATAL	2
+#define AUTH_INTR	2
+#define AUTH_FATAL	3
 
 typedef struct sudo_auth {
     short flags;		/* various flags, see below */
     short status;		/* status from verify routine */
     char *name;			/* name of the method as a string */
-    VOID *data;			/* method-specific data pointer */
+    void *data;			/* method-specific data pointer */
     int (*init) __P((struct passwd *pw, char **prompt, struct sudo_auth *auth));
     int (*setup) __P((struct passwd *pw, char **prompt, struct sudo_auth *auth));
     int (*verify) __P((struct passwd *pw, char *p, struct sudo_auth *auth));
@@ -57,6 +58,7 @@ int sia_setup __P((struct passwd *pw, char **prompt, sudo_auth *auth));
 int sia_verify __P((struct passwd *pw, char *prompt, sudo_auth *auth));
 int sia_cleanup __P((struct passwd *pw, sudo_auth *auth));
 int aixauth_verify __P((struct passwd *pw, char *pass, sudo_auth *auth));
+int aixauth_cleanup __P((struct passwd *pw, sudo_auth *auth));
 int bsdauth_init __P((struct passwd *pw, char **prompt, sudo_auth *auth));
 int bsdauth_verify __P((struct passwd *pw, char *prompt, sudo_auth *auth));
 int bsdauth_cleanup __P((struct passwd *pw, sudo_auth *auth));
@@ -92,7 +94,7 @@ int securid_verify __P((struct passwd *pw, char *pass, sudo_auth *auth));
 #  define AUTH_STANDALONE \
 	AUTH_ENTRY(0, "SecurId", \
 	    securid_init, securid_setup, securid_verify, NULL)
-#elif defined(HAVE_SIA)
+#elif defined(HAVE_SIA_SES_INIT)
 #  define AUTH_STANDALONE \
 	AUTH_ENTRY(0, "sia", \
 	    NULL, sia_setup, sia_verify, sia_cleanup)
